from wagtail.blocks.migrations.operations import BaseBlockOperation


class MergeLinkBlockOperation(BaseBlockOperation):
    """Merge a LinkBlock to the new LinkTargetBlock.

    The LinkBlock was initially a StuctBlock with one `target` child block which
    defines the link type and value through a LinkTargetBlock. Starting from the
    v0.5.0, the LinkBlock has been merged to LinkTargetBlock to simplify its
    structure and make it more reliable.

    Note:
        The `block_path_str` should point to the LinkBlock.
    """

    def apply(self, block_value):
        if targets := block_value.get("target"):
            return {
                "type": targets[0]["type"],
                targets[0]["type"]: targets[0]["value"],
            }

    @property
    def operation_name_fragment(self):
        return "merge_link_block"



from django.core.exceptions import ImproperlyConfigured

from wagtail.blocks.struct_block import StructBlockValidationError

import pytest
from bs4 import BeautifulSoup
from pytest_django.asserts import assertHTMLEqual
from wagtail_factories import DocumentFactory, ImageFactory, PageFactory

from wagtail_cblocks import blocks
from wagtail_cblocks.blocks import (
    CSSClassMixin,
    LinkTargetBlockAdapter,
    Style,
    StylizedStructBlock,
)

from .models import BaseBlock, ColumnsBlock, RowColumnsBlock


class BlockTest:
    def render(self, data, block=None):
        if block is None:
            block = self.block
        return block.render(block.to_python(data))

    def render_html(self, *args, **kwargs):
        return BeautifulSoup(self.render(*args, **kwargs), "html.parser")


class TestCSSClassMixin:
    class Block(CSSClassMixin, blocks.ParagraphBlock):
        pass

    def test_in_meta(self):
        class Block(self.Block):
            class Meta:
                css_class = "lead"

        block = Block()
        context = block.get_context(block.to_python("<p>Paragraph</p>"))
        assert context["css_class"] == "lead"

    @pytest.mark.parametrize("value", ("lead", {"lead"}, ["lead"]))
    def test_at_init(self, value):
        block = self.Block(css_class=value)
        context = block.get_context(block.to_python("<p>Paragraph</p>"))
        assert context["css_class"] == "lead"

    def test_empty(self):
        block = self.Block()
        context = block.get_context(block.to_python("<p>Paragraph</p>"))
        assert context["css_class"] == ""


class TestStylizedStructBlock:
    STYLES = [
        Style("foo", "Foo", "is-foo"),
        Style("bar", "Bar", "is-bar"),
    ]

    def test_empty_styles(self):
        block = StylizedStructBlock()
        assert "style" not in block.child_blocks
        context = block.get_context(block.to_python({}))
        assert context["css_class"] == ""

    def test_styles_from_property(self):
        class Block(StylizedStructBlock):
            styles = self.STYLES

        value = Block().to_python({"style": "foo"})
        assert value.get("style").name == "foo"

    def test_style(self):
        block = StylizedStructBlock(styles=self.STYLES)
        value = block.to_python({"style": "foo"})
        assert block.get_prep_value(value) == {"style": "foo"}
        assert isinstance(value["style"], Style)
        assert value["style"].name == "foo"
        context = block.get_context(value)
        assert context["css_class"] == "is-foo"

    def test_style_with_css_class(self):
        block = StylizedStructBlock(styles=self.STYLES, css_class="block")
        context = block.get_context(block.to_python({"style": "foo"}))
        assert context["css_class"] == "block is-foo"

    def test_default_style(self):
        block = StylizedStructBlock(styles=self.STYLES, default_style="bar")
        value = block.to_python({})
        assert isinstance(value.get("style"), Style)
        assert value.get("style").name == "bar"
        context = block.get_context(value)
        assert context["css_class"] == "is-bar"

    def test_default_style_object(self):
        block = StylizedStructBlock(
            styles=self.STYLES, default_style=self.STYLES[1]
        )
        value = block.to_python({})
        assert isinstance(value.get("style"), Style)
        assert value.get("style").name == "bar"

    def test_unknown_style(self):
        block = StylizedStructBlock(styles=self.STYLES, default_style="bar")
        value = block.to_python({"style": "baz"})
        assert value.get("style") is None
        context = block.get_context(value)
        assert context["css_class"] == ""


class TestHeadingBlock(BlockTest):
    block = blocks.HeadingBlock()

    def test_render(self):
        assertHTMLEqual(
            self.render({"text": "Un titre !", "level": 2}),
            '<h2 id="un-titre">Un titre !</h2>',
        )


class TestParagraphBlock(BlockTest):
    block = blocks.ParagraphBlock()

    def test_features(self):
        assert self.block.features == blocks.ParagraphBlock.features

        block = blocks.ParagraphBlock(features=["bold", "italic"])
        assert block.features == ["bold", "italic"]

    def test_render(self):
        data = "<p><i>Un</i> paragraphe !</p>"
        assertHTMLEqual(self.render(data), data)


@pytest.mark.django_db
class TestLinkTargetBlock:
    block = blocks.LinkTargetBlock()

    def get_value(self, link_type, value):
        return self.block.clean({"type": link_type, link_type: value})

    def test_not_required(self):
        block = blocks.LinkTargetBlock(required=False)
        assert block.required is False
        assert block.clean({}).href == ""

    def test_required(self):
        assert self.block.required is True

        with pytest.raises(StructBlockValidationError) as exc_info:
            self.block.clean({})
        assert exc_info.value.block_errors.keys() == {"type"}

    def test_value_page(self, root_page):
        page = PageFactory(parent=root_page, title="About", slug="about")
        value = self.get_value("page", page.id)
        assert value.href == "/about/"

    def test_clean_page_invalid(self):
        with pytest.raises(StructBlockValidationError) as exc_info:
            self.get_value("page", 1000)
        assert exc_info.value.block_errors.keys() == {"page"}

    def test_value_document(self):
        document = DocumentFactory()
        value = self.get_value("document", document.id)
        assert value.href == document.file.url

    def test_clean_document_invalid(self):
        with pytest.raises(StructBlockValidationError) as exc_info:
            self.get_value("document", 1000)
        assert exc_info.value.block_errors.keys() == {"document"}

    def test_value_image(self):
        image = ImageFactory()
        value = self.get_value("image", image.id)
        assert value.href == image.file.url

    def test_clean_image_invalid(self):
        with pytest.raises(StructBlockValidationError) as exc_info:
            self.get_value("image", 1000)
        assert exc_info.value.block_errors.keys() == {"image"}

    def test_value_external_url(self):
        url = "http://example.org/truc/"
        value = self.get_value("url", url)
        assert value.href == url

    def test_value_anchor(self):
        anchor = "#truc"
        value = self.get_value("anchor", anchor)
        assert value.href == anchor

    def test_clean_no_value(self):
        with pytest.raises(StructBlockValidationError) as exc_info:
            self.block.clean({"type": "page"})
        assert exc_info.value.block_errors.keys() == {"page"}

    def test_adapter(self):
        block = blocks.LinkTargetBlock()
        block.set_name("test_linkblock")

        js_args = LinkTargetBlockAdapter().js_args(block)
        assert js_args[0] == "test_linkblock"
        assert js_args[2]["required"] is True
        assert js_args[2]["blockTypes"] == [
            "page",
            "document",
            "image",
            "url",
            "anchor",
        ]
        assert "formTemplate" in js_args[2]


class TestButtonBlock(BlockTest):
    block = blocks.ButtonBlock()

    def test_render_link(self):
        url = "http://example.org/truc/"
        assertHTMLEqual(
            self.render(
                {
                    "text": "Lien",
                    "link": {"type": "url", "url": url},
                }
            ),
            f'<a class="btn mb-3" href="{url}">Lien</a>',
        )


@pytest.mark.django_db
class TestImageBlock(BlockTest):
    block = blocks.ImageBlock()

    def test_render(self):
        html = self.render_html(
            {
                "image": ImageFactory().id,
                "caption": "",
                "link": {},
            }
        )
        assert len(html.select("img.figure-img.img-fluid.mb-0")) == 1
        assert not html.select("figcaption")
        assert not html.select("a")

    def test_render_with_caption(self):
        html = self.render_html(
            {
                "image": ImageFactory().id,
                "caption": "Une légende en dessous.",
                "link": {},
            }
        )
        assert html.select_one("figcaption").text == "Une légende en dessous."
        assert not html.select("a")

    def test_render_with_link(self):
        url = "http://example.org/truc/"
        html = self.render_html(
            {
                "image": ImageFactory().id,
                "caption": "",
                "link": {"type": "url", "url": url},
            }
        )
        assert html.select_one("a").attrs["href"] == url
        assert not html.select("figcaption")


class TestColumnsBlock(BlockTest):
    block = blocks.ColumnsBlock(BaseBlock())

    def test_columns_block_order(self):
        assert list(self.block.child_blocks.keys())[0] == "columns"

    def test_render(self):
        url = "http://example.org/truc/"
        data = {
            "columns": [
                [
                    {
                        "type": "button_block",
                        "value": {
                            "text": "Lien",
                            "link": {"type": "url", "url": url},
                            "style": "primary",
                        },
                    },
                    {
                        "type": "paragraph_block",
                        "value": "<p>A first paragraph.</p>",
                    },
                ],
                [
                    {
                        "type": "paragraph_block",
                        "value": "<p>Another paragraph.</p>",
                    },
                ],
            ],
            "horizontal_align": "center",
        }
        assertHTMLEqual(
            self.render(data),
            (
                '<div class="row text-center">'
                '<div class="col-sm">{}{}</div>'
                '<div class="col-sm">{}</div>'
                "</div>"
            ).format(
                f'<a class="btn btn-primary mb-3" href="{url}">Lien</a>',
                "<p>A first paragraph.</p>",
                "<p>Another paragraph.</p>",
            ),
        )

    def test_sublcass_render(self):
        data = {
            "columns": [
                [
                    {
                        "type": "paragraph_block",
                        "value": "<p>A first paragraph.</p>",
                    },
                ],
            ],
            "layout": "auto",
        }
        assertHTMLEqual(
            self.render(data, RowColumnsBlock()),
            (
                '<div class="row row-cols-auto">'
                '<div class="col"><p>A first paragraph.</p></div>'
                "</div>"
            ),
        )

    def test_required_column_block(self):
        with pytest.raises(ImproperlyConfigured):

            class DummyColumnsBlock(ColumnsBlock):
                pass

            DummyColumnsBlock()



from django.urls import reverse

from wagtail.test.utils.form_data import (
    nested_form_data,
    rich_text,
    streamfield,
)

import pytest
from pytest_django.asserts import assertContains

from .models import StandardPage


@pytest.mark.django_db
def test_hero_block_preview(admin_client, root_page):
    add_url = reverse(
        "wagtailadmin_pages:add",
        args=("tests", "standardpage", root_page.id),
    )

    form_data = nested_form_data(
        {
            "title": "A page",
            "body": streamfield(
                [
                    (
                        "hero_block",
                        {
                            "style": "centered",
                            "blocks": streamfield(
                                [
                                    (
                                        "paragraph_block",
                                        rich_text("<p>Lorem ipsum</p>"),
                                    ),
                                ]
                            ),
                        },
                    ),
                ]
            ),
        }
    )
    form_data["slug"] = "a-page"
    response = admin_client.post(add_url, data=form_data)
    assert response.status_code == 302

    page = StandardPage.objects.get(slug="a-page")
    preview_url = reverse("wagtailadmin_pages:view_draft", args=(page.id,))

    response = admin_client.get(preview_url)
    assert response.status_code == 200
    assertContains(response, "hero-centered")
    assertContains(response, "Lorem ipsum")


@pytest.mark.django_db
def test_link_target_block_media(admin_client, root_page):
    add_url = reverse(
        "wagtailadmin_pages:add",
        args=("tests", "standardpage", root_page.id),
    )
    content = str(admin_client.get(add_url).content)
    assert "wagtail_cblocks/admin/css/link-target-block.css" in content
    assert "wagtail_cblocks/admin/js/link-target-block.js" in content



import os
from pathlib import Path

BASE_DIR = Path(__file__).parent
VAR_DIR = Path(__file__).parent / "var"

# GENERAL

DEBUG = True if os.environ.get("INTERACTIVE", "0") == "1" else False

SECRET_KEY = "not-a-secure-key"  # noqa: S105

ALLOWED_HOSTS = ["localhost", "testserver"]

# INTERNATIONALIZATION

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_TZ = True

# DATABASE

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": str(BASE_DIR / "sqlite.db"),
    }
}

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"

# URLS

ROOT_URLCONF = "tests.urls"

# APP CONFIGURATION

INSTALLED_APPS = [
    "wagtail_cblocks",
    "tests",
    # wagtail
    "wagtail.sites",
    "wagtail.users",
    "wagtail.documents",
    "wagtail.images",
    "wagtail.search",
    "wagtail.admin",
    "wagtail",
    "modelcluster",
    "taggit",
    # django
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]

# PASSWORDS

PASSWORD_HASHERS = ["django.contrib.auth.hashers.MD5PasswordHasher"]

# MIDDLEWARE

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "wagtail.contrib.redirects.middleware.RedirectMiddleware",
]

# STATIC

STATIC_ROOT = VAR_DIR / "static"

STATIC_URL = "/static/"

STATICFILES_FINDERS = [
    "django.contrib.staticfiles.finders.FileSystemFinder",
    "django.contrib.staticfiles.finders.AppDirectoriesFinder",
]

# MEDIA

MEDIA_ROOT = VAR_DIR / "media"

MEDIA_URL = "/media/"

# TEMPLATES

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

# CACHES

CACHES = {
    "default": {
        "BACKEND": "django.core.cache.backends.locmem.LocMemCache",
        "LOCATION": "",
    }
}

# WAGTAIL

WAGTAILADMIN_BASE_URL = "http://testserver"

WAGTAIL_SITE_NAME = "wagtail-cblocks test"



from collections import namedtuple

from django import forms
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.utils.functional import cached_property
from django.utils.safestring import mark_safe
from django.utils.text import slugify
from django.utils.translation import gettext_lazy as _

from wagtail import blocks as wagtail_blocks
from wagtail.blocks.struct_block import (
    StructBlockAdapter,
    StructBlockValidationError,
)
from wagtail.documents.blocks import DocumentChooserBlock
from wagtail.images.blocks import ImageChooserBlock
from wagtail.telepath import register

# MIXINS AND BASE CLASSES
# ------------------------------------------------------------------------------

Style = namedtuple("Style", ["name", "label", "css_class"])


class CSSClassMixin:
    """
    Add the CSS class defined for a block to its rendering context.
    """

    def get_css_classes(self, value):
        if hasattr(self.meta, "css_class"):
            if isinstance(self.meta.css_class, str):
                return [self.meta.css_class]
            if not isinstance(self.meta.css_class, list):
                return list(self.meta.css_class)
            return self.meta.css_class
        return []

    def get_context(self, value, **kwargs):
        context = super().get_context(value, **kwargs)
        context["css_class"] = " ".join(self.get_css_classes(value))
        return context


class StyleChoiceBlock(wagtail_blocks.ChoiceBlock):
    class Meta:
        label = _("Style")

    def __init__(self, styles, default=None):
        self.styles = styles

        super().__init__(
            choices=[(s.name, s.label) for s in styles],
            default=default,
        )

    def to_python(self, value):
        return self.get_style_by_name(value)

    def get_prep_value(self, value):
        return value.name

    def value_from_form(self, value):
        return self.get_style_by_name(value)

    def value_for_form(self, value):
        return value and value.name

    def get_default(self):
        if isinstance(self.meta.default, Style):
            return self.meta.default
        return self.to_python(self.meta.default)

    def get_style_by_name(self, name):
        for style in self.styles:
            if style.name == name:
                return style


class StylizedStructBlock(CSSClassMixin, wagtail_blocks.StructBlock):
    """
    A group of sub-blocks which defines an element with different styles.
    """

    styles = ()

    class Meta:
        default_style = None

    def __init__(self, *args, styles=None, **kwargs):
        super().__init__(*args, **kwargs)

        if styles is None:
            styles = self.styles

        if styles:
            style = StyleChoiceBlock(styles, default=self.meta.default_style)
            style.set_name("style")

            self.child_blocks["style"] = style

    def get_css_classes(self, value):
        css_classes = super().get_css_classes(value)
        style = value.get("style", None)
        if style and style.css_class:
            css_classes.append(style.css_class)
        return css_classes


# TYPOGRAPHY
# ------------------------------------------------------------------------------

HEADING_LEVELS = [
    (i, _("Level %(level)d heading") % {"level": i}) for i in range(2, 6)
]


class HeadingValue(wagtail_blocks.StructValue):
    @cached_property
    def tag(self):
        """Return the HTML tag to use for the level."""
        return "h{}".format(self.get("level"))

    @cached_property
    def anchor(self):
        """Generate a slug from the title to be used as an anchor."""
        return slugify(self.get("text"))


class HeadingBlock(wagtail_blocks.StructBlock):
    """
    A section heading with a choosable level.
    """

    text = wagtail_blocks.CharBlock(label=_("Title"), classname="title")
    level = wagtail_blocks.ChoiceBlock(
        choices=HEADING_LEVELS,
        default=2,
        label=_("Level"),
    )

    class Meta:
        icon = "title"
        label = _("Title")
        template = "wagtail_cblocks/heading_block.html"
        value_class = HeadingValue


class ParagraphBlock(wagtail_blocks.RichTextBlock):
    """
    A paragraph with simple or customized features.
    """

    features = ("bold", "italic", "ol", "ul", "hr", "link", "document-link")

    def __init__(self, features=None, **kwargs):
        if features is None:
            features = self.features
        return super().__init__(features=features, **kwargs)

    class Meta:
        icon = "pilcrow"
        label = _("Paragraph")
        template = "wagtail_cblocks/paragraph_block.html"


# LINK
# ------------------------------------------------------------------------------


class LinkTargetStructValue(wagtail_blocks.StructValue):
    @cached_property
    def href(self):
        if block_type := self.get("type"):
            value = self.get(block_type)
            if block_type == "page":
                return value.url
            if block_type in ("document", "image"):
                return value.file.url
            return value
        return ""


class LinkTargetBlock(wagtail_blocks.StructBlock):
    class Meta:
        icon = "link"
        label = _("Link")
        value_class = LinkTargetStructValue
        form_template = "wagtail_cblocks/admin/forms/link_target_block.html"

    page = wagtail_blocks.PageChooserBlock(required=False, label=_("Page"))
    document = DocumentChooserBlock(required=False, label=_("Document"))
    image = ImageChooserBlock(required=False, label=_("Image"))
    url = wagtail_blocks.URLBlock(required=False, label=_("External link"))
    anchor = wagtail_blocks.CharBlock(
        required=False,
        label=_("Anchor link"),
        help_text=mark_safe(  # noqa: S308
            _(
                "An anchor in the current page, for example: "
                "<code>#target-id</code>."
            )
        ),
    )

    def __init__(self, local_blocks=None, required=True, **kwargs):
        super().__init__(local_blocks=local_blocks, **kwargs)

        self.meta.required = required

        # Retrieve available block types from the defined blocks
        self.block_types = list(self.child_blocks.keys())

        # Construct dynamically the choice block and append it
        type_block = wagtail_blocks.ChoiceBlock(
            choices=[
                (name, self.child_blocks[name].label)
                for name in self.block_types
            ]
        )
        type_block.set_name("type")

        self.child_blocks["type"] = type_block

    @property
    def required(self):
        return self.meta.required

    def clean(self, value):
        # Build up a list of (name, value) tuples to be passed to the
        # StructValue constructor
        result = []

        errors = {}

        if block_type := value.get("type"):
            result.append(("type", block_type))

            if block_value := value.get(block_type):
                try:
                    result.append(
                        (
                            block_type,
                            self.child_blocks[block_type].clean(block_value),
                        )
                    )
                except ValidationError as e:
                    errors[block_type] = e
            else:
                errors[block_type] = ValidationError(
                    _("This field is required."), code="required"
                )
        elif self.required:
            errors["type"] = ValidationError(
                _("This field is required."), code="required"
            )

        if errors:
            raise StructBlockValidationError(errors)

        return self._to_struct_value(result)


class LinkTargetBlockAdapter(StructBlockAdapter):
    js_constructor = "wagtail_cblocks.blocks.LinkTargetBlock"

    def js_args(self, block):
        js_args = super().js_args(block)
        js_args[2]["blockTypes"] = block.block_types
        return js_args

    @cached_property
    def media(self):
        css = super().media._css
        css.setdefault("all", [])
        css["all"].append("wagtail_cblocks/admin/css/link-target-block.css")

        js = super().media._js
        js.append("wagtail_cblocks/admin/js/link-target-block.js")

        return forms.Media(css=css, js=js)


register(LinkTargetBlockAdapter(), LinkTargetBlock)


# BUTTONS
# ------------------------------------------------------------------------------


class ButtonBlock(StylizedStructBlock):
    """
    A button which acts like a link.
    """

    text = wagtail_blocks.CharBlock(label=_("Text"))
    link = LinkTargetBlock()

    class Meta:
        icon = "link"
        label = _("Button")
        template = "wagtail_cblocks/button_block.html"


# IMAGES
# ------------------------------------------------------------------------------


class ImageBlock(wagtail_blocks.StructBlock):
    """
    An image with optional caption and link.
    """

    image = ImageChooserBlock(label=_("Image"))
    caption = wagtail_blocks.CharBlock(required=False, label=_("Caption"))
    link = LinkTargetBlock(required=False)

    class Meta:
        icon = "image"
        label = _("Image")
        template = "wagtail_cblocks/image_block.html"


# LAYOUT
# ------------------------------------------------------------------------------

HORIZONTAL_ALIGNMENTS = [
    ("start", _("Left")),
    ("center", _("Center")),
    ("end", _("Right")),
]
HORIZONTAL_ALIGNMENT_DEFAULT = None


class ColumnsBlock(wagtail_blocks.StructBlock):
    """
    A list of columns which can be horizontally aligned.
    """

    horizontal_align = wagtail_blocks.ChoiceBlock(
        choices=HORIZONTAL_ALIGNMENTS,
        default=HORIZONTAL_ALIGNMENT_DEFAULT,
        required=False,
        label=_("Horizontal alignment"),
    )

    class Meta:
        icon = "table"
        label = _("Columns")
        template = "wagtail_cblocks/columns_block.html"

    def __init__(self, column_block=None, **kwargs):
        super().__init__(**kwargs)

        if column_block is None:
            if not hasattr(self.meta, "column_block"):
                raise ImproperlyConfigured(
                    "ColumnsBlock was not passed a 'column_block' object"
                )
            column_block = self.meta.column_block

        columns = wagtail_blocks.ListBlock(
            column_block,
            collapsed=True,
            form_classname="columns-block-list",
            label=_("Columns"),
        )
        columns.set_name("columns")

        self.child_blocks["columns"] = columns
        self.child_blocks.move_to_end("columns", last=False)



from wagtail_cblocks.blocks.migrations.operations import MergeLinkBlockOperation


class TestMergeLinkBlockOperation:
    def test_init(self):
        operation = MergeLinkBlockOperation()
        assert operation.operation_name_fragment == "merge_link_block"

    def test_no_value(self):
        operation = MergeLinkBlockOperation()
        assert operation.apply({}) is None
        assert operation.apply({"target": []}) is None

    def test_migrate(self):
        operation = MergeLinkBlockOperation()
        assert operation.apply(
            {
                "target": [
                    {
                        "type": "anchor",
                        "value": "#pouet",
                        "id": "34a9cf54-3b56-4d10-8f8a-172ca33f57b3",
                    }
                ],
            }
        ) == {
            "type": "anchor",
            "anchor": "#pouet",
        }



from wagtail.admin.panels import FieldPanel
from wagtail.blocks import ChoiceBlock, StreamBlock
from wagtail.fields import StreamField
from wagtail.models import Page

from wagtail_cblocks.blocks import (
    ButtonBlock,
    ColumnsBlock,
    HeadingBlock,
    ImageBlock,
    ParagraphBlock,
    Style,
    StylizedStructBlock,
)

BUTTON_STYLES = [
    Style("primary", "Primary", "btn-primary"),
    Style("secondary", "Secondary", "btn-secondary"),
    Style("primary-lg", "Large primary", "btn-primary btn-lg"),
]
BUTTON_DEFAULT_STYLE = "primary-lg"


class BaseBlock(StreamBlock):
    title_block = HeadingBlock()
    paragraph_block = ParagraphBlock()
    button_block = ButtonBlock(
        styles=BUTTON_STYLES, default_style=BUTTON_DEFAULT_STYLE
    )
    image_block = ImageBlock()


class RowColumnsBlock(ColumnsBlock):
    LAYOUT_CHOICES = [
        ("2", "2-columns"),
        ("3", "3-columns"),
        ("4", "4-columns"),
        ("auto", "natural width"),
    ]

    layout = ChoiceBlock(choices=LAYOUT_CHOICES, default="auto")

    class Meta:
        label = "Row columns"
        template = "tests/blocks/row_columns_block.html"
        column_block = BaseBlock()


class HeroBlock(StylizedStructBlock):
    blocks = BaseBlock(
        local_blocks=[("columns_block", ColumnsBlock(BaseBlock()))],
        label="Content",
    )

    styles = [
        Style("centered", "Centered", "my-5 text-center hero-centered"),
        Style("responsive", "Responsive", "container col-xxl-8"),
        Style("dark", "Dark", "bg-dark text-white text-center"),
    ]

    class Meta:
        label = "Hero"
        template = "tests/blocks/hero_block.html"


class BodyBlock(BaseBlock):
    hero_block = HeroBlock()
    columns_block = ColumnsBlock(BaseBlock())
    row_columns_block = RowColumnsBlock()


class StandardPage(Page):
    body = StreamField(BodyBlock(), use_json_field=True)

    content_panels = Page.content_panels + [
        FieldPanel("body"),
    ]
